#!/bin/bash
# Script bash to copy the needed compiler library for the Ac6 Rootfs Project
# Do executed on the project root directory
#
# REQUIRED VARIABLES :
# compiler_prefix : the GCC-based compiler prefix
#   
# Do not modify !

here=$(pwd)
basedir=$(dirname "$0")
rootfsDir=$here/output/rootfs
sysrootDir=$SYSROOT_LOC

bash="/bin/bash"
sed="/bin/sed"
grep="/bin/grep"
readlink="/bin/readlink"
crossldd="$basedir/cross-ldd"
 
prefix=$compiler_prefix
readelf="/usr/bin/readelf"

declare -a sourceDirs=("bin" "usr/bin" "usr/lib" "usr/sbin" "lib")
declare -a listElf

mkdir -p ${rootfsDir}/lib

# Check if the readelf or cross-readelf is existing
if [ ! -x "$( which "${readelf}" )" ]; then
	readelf="${prefix}-readelf"
fi
if [ ! -x "$( which "${readelf}" )" ]; then
	echo "Can't execute 'readelf' nor '${readelf}'"
	exit 1
fi

# Check if the file in argument is an ELF file then add it in a list
add_elf() {
	local elfFile=$1
	local type=$( ${readelf} -h ${elfFile} 2> /dev/null | ${grep} Class | ${sed} -r -e 's/^.*Class.*:[[:space:]]+(.*)$/\1/;' )
	if [[ $type == *"ELF"* ]]; then
		listElf=(${listElf[@]} ${elfFile})
	fi
}

# Get the list of ELF files from the input directory 
get_elf_files() {	
	local dir=$1	
	# If the source directory exists		
	if [ -d $dir ]; then
		for file in $( find $dir -type f )
		do			
			add_elf $file
		done
	fi
}

# Copy the lib C if does not exist
copy_libc() {
	local dest=$1
	local source=$2
	local linkSource="$(${readlink} -f ${source})"

	if [ ! -s "${dest}" ]; then
		if [ -s "${linkSource}" ]; then
			destLib="$(dirname $dest)/$(basename $linkSource)"
			destLink=${dest}
			echo "cp -a ${linkSource} ${destLib}"
			echo "ln -nfs ${destLib} ${destLink}"
			cp -a ${linkSource} ${destLib}
			ln -nfs ${destLib} ${destLink}
		else
			echo "cp -a ${source} ${dest}"
			cp -a ${source} ${dest}
		fi	
	fi
}

# Copy the required C libraries for the file in argument
copy_libsc() {
	local elfFile=$1
	${bash} ${crossldd} -m --prefix=$prefix --root=$rootfsDir $elfFile | ${sed} 1d | \
	while read line
	do
		if [ ! -z "${line}" ]; then
			source=$(echo $line | awk -F' <= ' '{ print $1 }')	
			dest=$(echo $line | awk -F' <= ' '{ print $2 }')
			copy_libc $source $dest
		fi
	done
}

# If the cross-ldd script exists
if [ -f ${crossldd} ]; then
	
 	# Search every ELF files from source directories
	for sourceDir in ${sourceDirs[*]}
	do
		dir=$rootfsDir/$sourceDir
		get_elf_files $dir
	done

	for elfFile in ${listElf[*]}
	do	
		copy_libsc $elfFile
	done
fi


